@@ -13,6 +13,7 @@ from __future__ import annotations
 import collections
 import contextlib
 import functools
+import io
 import itertools
 import json
 import os
@@ -23,6 +24,7 @@ import shutil
 import subprocess
 import sys
 import sysconfig
+import tarfile
 import tempfile
 import textwrap
 import typing
@@ -48,13 +50,12 @@ if typing.TYPE_CHECKING:  # pragma: no cover
     import wheel.wheelfile  # noqa: F401
 
 
-__version__ = '0.4.0'
+__version__ = '0.5.0'
 
 
 class _depstr:
     ninja = 'ninja >= 1.10.0'
-    patchelf_wrapper = 'patchelf-wrapper'
-    pep621 = 'pep621 >= 0.3.0'
+    patchelf = 'patchelf >= 0.11.0'
     wheel = 'wheel >= 0.36.0'  # noqa: F811
 
 
@@ -539,7 +540,8 @@ class Project():
         assert isinstance(version, str)
         return version
 
-    @property
+    @property  # type: ignore[misc]
+    @functools.lru_cache(maxsize=1)
     def metadata(self) -> bytes:  # noqa: C901
         """Project metadata."""
         # the rest of the keys are only available when using PEP 621 metadata
@@ -719,26 +721,49 @@ class Project():
     def sdist(self, directory: Path) -> pathlib.Path:
         """Generates a sdist (source distribution) in the specified directory."""
         # generate meson dist file
-        self._meson('dist', '--no-tests', '--formats', 'gztar')
+        self._meson('dist', '--allow-dirty', '--no-tests', '--formats', 'gztar')
 
         # move meson dist file to output path
         dist_name = f'{self.name}-{self.version}'
         meson_dist_name = f'{self._meson_name}-{self._meson_version}'
-        meson_dist = pathlib.Path(self._build_dir, 'meson-dist', f'{meson_dist_name}.tar.gz')
+        meson_dist_path = pathlib.Path(self._build_dir, 'meson-dist', f'{meson_dist_name}.tar.gz')
         sdist = pathlib.Path(directory, f'{dist_name}.tar.gz')
 
-        with mesonpy._util.edit_targz(meson_dist, sdist) as content:
-            # rename from meson name to sdist name if necessary
-            if dist_name != meson_dist_name:
-                shutil.move(str(content / meson_dist_name), str(content / dist_name))
+        with tarfile.open(meson_dist_path, 'r:gz') as meson_dist, mesonpy._util.create_targz(sdist) as (tar, mtime):
+            for member in meson_dist.getmembers():
+                # skip the generated meson native file
+                if member.name == f'{meson_dist_name}/.mesonpy-native-file.ini':
+                    continue
 
-            # remove .mesonpy-native-file.ini if it exists
-            native_file = content / meson_dist_name / '.mesonpy-native-file.ini'
-            if native_file.exists():
-                native_file.unlink()
+                # calculate the file path in the source directory
+                assert member.name, member.name
+                member_parts = member.name.split('/')
+                if len(member_parts) <= 1:
+                    continue
+                path = self._source_dir.joinpath(*member_parts[1:])
+
+                if not path.is_file():
+                    continue
+
+                info = tarfile.TarInfo(member.name)
+                info.size = os.path.getsize(path)
+
+                # rewrite the path if necessary, to match the sdist distribution name
+                if dist_name != meson_dist_name:
+                    info.name = pathlib.Path(
+                        dist_name,
+                        path.relative_to(self._source_dir)
+                    ).as_posix()
+
+                with path.open('rb') as f:
+                    tar.addfile(info, fileobj=f)
 
             # add PKG-INFO to dist file to make it a sdist
-            content.joinpath(dist_name, 'PKG-INFO').write_bytes(self.metadata)
+            pkginfo_info = tarfile.TarInfo(f'{dist_name}/PKG-INFO')
+            if mtime:
+                pkginfo_info.mtime = mtime
+            pkginfo_info.size = len(self.metadata)  # type: ignore[arg-type]
+            tar.addfile(pkginfo_info, fileobj=io.BytesIO(self.metadata))  # type: ignore[arg-type]
 
         return sdist
 
@@ -762,16 +787,6 @@ def _project(config_settings: Optional[Dict[Any, Any]]) -> Iterator[Project]:
         yield project
 
 
-def get_requires_for_build_sdist(
-    config_settings: Optional[Dict[Any, Any]] = None,
-) -> List[str]:
-    dependencies = []
-    with _project(config_settings) as project:
-        if project.pep621:
-            dependencies.append(_depstr.pep621)
-    return dependencies
-
-
 def build_sdist(
     sdist_directory: str,
     config_settings: Optional[Dict[Any, Any]] = None,
@@ -789,9 +804,10 @@ def get_requires_for_build_wheel(
     dependencies = [_depstr.wheel, _depstr.ninja]
     with _project(config_settings) as project:
         if not project.is_pure and platform.system() == 'Linux':
-            dependencies.append(_depstr.patchelf_wrapper)
-        if project.pep621:
-            dependencies.append(_depstr.pep621)
+            # we may need patchelf
+            if not shutil.which('patchelf'):  # XXX: This is slightly dangerous.
+                # patchelf not already acessible on the system
+                dependencies.append(_depstr.patchelf)
     return dependencies
 
 
